@@ -42,10 +42,12 @@ class DataTypeTest(TestCase):
       properties:
         addresses:
           type: map
+          required: false
           entry_schema:
             type: string
         contacts:
           type: list
+          required: false
           entry_schema:
             type: tosca.my.datatypes.ContactInfo
 
@@ -61,6 +63,24 @@ class DataTypeTest(TestCase):
           type: string
         contact_phone:
           type: string
+
+    tosca.my.datatypes.TestLab:
+      properties:
+        humidity:
+          type: range
+          required: false
+          constraints:
+            - in_range: [-256, INFINITY]
+        temperature1:
+          type: range
+          required: false
+          constraints:
+            - in_range: [-256, UNBOUNDED]
+        temperature2:
+          type: range
+          required: false
+          constraints:
+            - in_range: [UNBOUNDED, 256]
     '''
     custom_type_def = yamlparser.simple_parse(custom_type_schema)
 
@@ -116,6 +136,31 @@ class DataTypeTest(TestCase):
         data = DataEntity('PortInfo', value.get('ethernet_port'))
         self.assertIsNotNone(data.validate())
 
+    # Test normative PortSpec datatype's additional requirements
+    # TODO(Matt) - opened as bug 1555300
+    # Need a test for PortSpec normative data type
+    # that tests the spec. requirement: "A valid PortSpec
+    # must have at least one of the following properties:
+    # target, target_range, source or source_range."
+    # TODO(Matt) - opened as bug 1555310
+    # test PortSpec value for source and target
+    # against the source_range and target_range
+    # when specified.
+    def test_port_spec_addl_reqs(self):
+        value_snippet = '''
+        test_port:
+          protocol: tcp
+          target: 65535
+          target_range: [ 1, 65535 ]
+          source: 1
+          source_range: [ 1, 65535 ]
+
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('tosca.datatypes.network.PortSpec',
+                          value.get('test_port'))
+        self.assertIsNotNone(data.validate())
+
     def test_built_in_datatype_without_properties(self):
         value_snippet = '''
         2
@@ -274,7 +319,7 @@ class DataTypeTest(TestCase):
         error = self.assertRaises(ValueError, data.validate)
         self.assertEqual(_('"1" is not a string.'), error.__str__())
 
-    # contact_pone is an invalid field name in nested datatype
+    # 'contact_pone' is an invalid attribute name in nested datatype below
     def test_validation_in_nested_datatype(self):
         value_snippet = '''
         name: Mike
@@ -325,3 +370,148 @@ class DataTypeTest(TestCase):
         self.assertRaises(exception.ValidationError, ToscaTemplate, tpl_path)
         exception.ExceptionCollector.assertExceptionMessage(
             ValueError, _('"123456789" is not a string.'))
+
+    def test_valid_range_type(self):
+        value_snippet = '''
+        user_port:
+          protocol: tcp
+          target_range:  [20000, 60000]
+          source_range:  [1000, 3000]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('PortSpec', value.get('user_port'))
+        self.assertIsNotNone(data.validate())
+
+    def test_invalid_range_datatype(self):
+        value_snippet = '''
+        user_port:
+          protocol: tcp
+          target: 1
+          target_range: [20000]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('PortSpec', value.get('user_port'))
+        err = self.assertRaises(ValueError, data.validate)
+        self.assertEqual(_('"[20000]" is not a valid range.'
+                           ),
+                         err.__str__())
+
+        value_snippet = '''
+        user_port:
+          protocol: tcp
+          target: 1
+          target_range: [20000, 3000]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('PortSpec', value.get('user_port'))
+        err = self.assertRaises(ValueError, data.validate)
+        self.assertEqual(_('"[20000, 3000]" is not a valid range.'
+                           ),
+                         err.__str__())
+
+        value_snippet = '''
+        humidity: [-100, 100]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('tosca.my.datatypes.TestLab',
+                          value, DataTypeTest.custom_type_def)
+        err = self.assertRaises(exception.InvalidSchemaError,
+                                lambda: data.validate())
+        self.assertEqual(_('The property "in_range" expects comparable values.'
+                           ),
+                         err.__str__())
+
+    def test_range_unbounded(self):
+        value_snippet = '''
+        humidity: [-100, 100]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('tosca.my.datatypes.TestLab',
+                          value, DataTypeTest.custom_type_def)
+        err = self.assertRaises(exception.InvalidSchemaError,
+                                lambda: data.validate())
+        self.assertEqual(_('The property "in_range" expects comparable values.'
+                           ),
+                         err.__str__())
+
+    def test_invalid_ranges_against_constraints(self):
+        # The TestLab range type has min=-256, max=UNBOUNDED
+        value_snippet = '''
+        temperature1: [-257, 999999]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('tosca.my.datatypes.TestLab', value,
+                          DataTypeTest.custom_type_def)
+        err = self.assertRaises(exception.ValidationError, data.validate)
+        self.assertEqual(_('The value "-257" of property "temperature1" is '
+                           'out of range "(min:-256, max:UNBOUNDED)".'),
+                         err.__str__())
+
+        value_snippet = '''
+        temperature2: [-999999, 257]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('tosca.my.datatypes.TestLab', value,
+                          DataTypeTest.custom_type_def)
+        err = self.assertRaises(exception.ValidationError, data.validate)
+        self.assertEqual(_('The value "257" of property "temperature2" is '
+                           'out of range "(min:UNBOUNDED, max:256)".'),
+                         err.__str__())
+
+    def test_valid_ranges_against_constraints(self):
+
+        # The TestLab range type has max=UNBOUNDED
+        value_snippet = '''
+        temperature1: [-255, 999999]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('tosca.my.datatypes.TestLab', value,
+                          DataTypeTest.custom_type_def)
+        self.assertIsNotNone(data.validate())
+
+        # The TestLab range type has min=UNBOUNDED
+        value_snippet = '''
+        temperature2: [-999999, 255]
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('tosca.my.datatypes.TestLab', value,
+                          DataTypeTest.custom_type_def)
+        self.assertIsNotNone(data.validate())
+
+    def test_incorrect_field_in_datatype(self):
+        tpl_snippet = '''
+        tosca_definitions_version: tosca_simple_yaml_1_0
+        topology_template:
+          node_templates:
+            server:
+              type: tosca.nodes.Compute
+
+            webserver:
+              type: tosca.nodes.WebServer
+              properties:
+                admin_credential:
+                  user: username
+                  token: some_pass
+                  some_field: value
+              requirements:
+                - host: server
+        '''
+        tpl = yamlparser.simple_parse(tpl_snippet)
+        err = self.assertRaises(exception.ValidationError, ToscaTemplate,
+                                None, None, None, tpl)
+        self.assertIn(_('The pre-parsed input failed validation with the '
+                        'following error(s): \n\n\tUnknownFieldError: Data '
+                        'value of type "tosca.datatypes.Credential" contains'
+                        ' unknown field "some_field". Refer to the definition'
+                        ' to verify valid values'), err.__str__())
+
+    def test_functions_datatype(self):
+        value_snippet = '''
+        admin_credential:
+          user: username
+          token: { get_input: password }
+        '''
+        value = yamlparser.simple_parse(value_snippet)
+        data = DataEntity('tosca.datatypes.Credential',
+                          value.get('admin_credential'))
+        self.assertIsNotNone(data.validate())
